@@ -1,7 +1,6 @@
 import dataclasses
 import sys
 import typing
-from dataclasses import dataclass
 from typing import Optional, Union
 
 try:
@@ -10,34 +9,62 @@ except ImportError:
     from typing_extensions import Annotated
 
 
+def _compact_dataclass_repr(obj) -> str:
+    arglist = ", ".join(
+        repr(getattr(obj, field.name)) for field in dataclasses.fields(obj)
+    )
+    return f"{obj.__class__.__name__}({arglist})"
+
+
 class CompactDataClass:
     "A data class whose repr() uses positional rather than keyword arguments."
 
     def __repr__(self) -> str:
-        arglist = ", ".join(
-            repr(getattr(self, field.name)) for field in dataclasses.fields(self)
-        )
-        return f"{self.__class__.__name__}({arglist})"
+        return _compact_dataclass_repr(self)
 
 
-@dataclass(frozen=True, repr=False)
-class Signed(CompactDataClass):
+def typeannotation(cls=None, /, *, eq=True, order=False):
+    "Returns the same class as was passed in, with dunder methods added based on the fields defined in the class."
+
+    data_cls = dataclasses.dataclass(
+        cls,
+        init=True,
+        repr=True,
+        eq=eq,
+        order=order,
+        unsafe_hash=False,
+        frozen=True,
+    )
+    data_cls.__repr__ = _compact_dataclass_repr
+    return data_cls
+
+
+@typeannotation
+class Signed:
+    "Signedness of an integer type."
+
     is_signed: bool
 
 
-@dataclass(frozen=True, repr=False)
-class Storage(CompactDataClass):
+@typeannotation
+class Storage:
+    "Number of bytes the binary representation of an integer type takes, e.g. 4 bytes for an int32."
+
     bytes: int
 
 
-@dataclass(frozen=True, repr=False)
-class IntegerRange(CompactDataClass):
+@typeannotation
+class IntegerRange:
+    "Minimum and maximum value of an integer. The range is inclusive."
+
     minimum: int
     maximum: int
 
 
-@dataclass(frozen=True, repr=False)
-class Precision(CompactDataClass):
+@typeannotation
+class Precision:
+    "Precision of a floating-point value."
+
     significant_digits: int
     decimal_digits: Optional[int] = 0
 
@@ -46,16 +73,25 @@ class Precision(CompactDataClass):
         return self.significant_digits - self.decimal_digits
 
 
-@dataclass(frozen=True, repr=False)
-class MinLength(CompactDataClass):
+@typeannotation
+class MinLength:
+    "Minimum length of a string."
+
     value: int
 
 
-@dataclass(frozen=True, repr=False)
-class MaxLength(CompactDataClass):
+@typeannotation
+class MaxLength:
+    "Maximum length of a string."
+
     value: int
 
 
+@typeannotation
+class SpecialConversion:
+    "Indicates that the annotated type is subject to custom conversion rules."
+
+
 int8 = Annotated[int, Signed(True), Storage(1), IntegerRange(-128, 127)]
 int16 = Annotated[int, Signed(True), Storage(2), IntegerRange(-32768, 32767)]
 int32 = Annotated[int, Signed(True), Storage(4), IntegerRange(-2147483648, 2147483647)]
@@ -94,7 +130,7 @@ def _python_type_to_str(data_type: type) -> str:
 
     origin = typing.get_origin(data_type)
     if origin is not None:
-        args = ", ".join(python_type_to_str(t) for t in typing.get_args(data_type))
+        data_type_args = typing.get_args(data_type)
 
         if origin is dict:  # Dict[T]
             origin_name = "Dict"
@@ -103,10 +139,16 @@ def _python_type_to_str(data_type: type) -> str:
         elif origin is set:  # Set[T]
             origin_name = "Set"
         elif origin is Union:
-            origin_name = "Union"
+            if type(None) in data_type_args:
+                # Optional[T] is represented as Union[T, None]
+                origin_name = "Optional"
+                data_type_args = tuple(t for t in data_type_args if t is not type(None))
+            else:
+                origin_name = "Union"
         else:
             origin_name = origin.__name__
 
+        args = ", ".join(python_type_to_str(t) for t in data_type_args)
         return f"{origin_name}[{args}]"
 
     if isinstance(data_type, typing.ForwardRef):
@@ -119,6 +161,9 @@ def _python_type_to_str(data_type: type) -> str:
 def python_type_to_str(data_type: type) -> str:
     "Returns the string representation of a Python type."
 
+    if data_type is type(None):
+        return "None"
+
     # use compact name for alias types
     name = _auxiliary_types.get(data_type)
     if name is not None:
