@@ -1,8 +1,4 @@
-from typing import (
-    Dict,
-    List,
-    Union,
-)
+from typing import Dict, List, Union
 
 
 class JsonObject:
